package sjtu.wcap.findyourfriendserver;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.List;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.MotionEvent;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

public class FindYourFriendServerActivity extends MapActivity {

	private static MapView mapView;
	private static MapController mc;
	private static GeoPoint p;
	private static GeoPoint pFriend;

	private LocationManager lm;
	private TextView mText;
	private static ServerSocket server = null;
	private static Socket client = null;
	private static DataOutputStream dataOutputStream = null;
	private static DataInputStream dataInputStream = null;
	private static boolean flag = true;
	private static boolean flagConnectServer = false;
	private static boolean flagConnectClient = false;

	class MapOverlay extends com.google.android.maps.Overlay {

		public boolean draw(Canvas canvas, MapView mapView, boolean shadow,
				long when) {
			Paint paint = new Paint();

			super.draw(canvas, mapView, shadow);
			// Converts lat/lng-Point to OUR coordinates on the screen.
			Point myScreenCoords = new Point();
			mapView.getProjection().toPixels(p, myScreenCoords);

			paint.setStrokeWidth(1);
			paint.setARGB(127, 255, 0, 0);
			paint.setStyle(Paint.Style.STROKE);

			Bitmap bmp = BitmapFactory.decodeResource(getResources(),
					R.drawable.pushpin);

			canvas.drawBitmap(bmp, myScreenCoords.x, myScreenCoords.y, paint);
			canvas.drawText("I am here...", myScreenCoords.x, myScreenCoords.y,
					paint);

			Point friendScreenCoords = new Point();
			mapView.getProjection().toPixels(pFriend, friendScreenCoords);

			paint.setARGB(127, 255, 0, 0);
			paint.setStyle(Paint.Style.STROKE);

			Bitmap bmpFriend = BitmapFactory.decodeResource(getResources(),
					R.drawable.marker);

			canvas.drawBitmap(bmpFriend, friendScreenCoords.x,
					friendScreenCoords.y, paint);
			canvas.drawText("My friend is there...", friendScreenCoords.x,
					friendScreenCoords.y, paint);

			return true;
		}

		@Override
		public boolean onTouchEvent(MotionEvent event, MapView mapView) {
			// ---when user lifts his finger---
			if (event.getAction() == 1) {
				GeoPoint p = mapView.getProjection().fromPixels(
						(int) event.getX(), (int) event.getY());
				Toast.makeText(
						getBaseContext(),
						"Location: " + p.getLatitudeE6() / 1E6 + ",|"
								+ p.getLongitudeE6() / 1E6, Toast.LENGTH_SHORT)
						.show();
			}
			return false;
		}
	}

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		// Displaying Zooming controls
		mapView = (MapView) findViewById(R.id.mapView);
		mapView.setBuiltInZoomControls(true);
		mapView.setTraffic(true);

		mText = (TextView) this.findViewById(R.id.text);

		mc = mapView.getController();
		String coordinates[] = { "31.035780", "121.438100" };
		double lat = Double.parseDouble(coordinates[0]);
		double lng = Double.parseDouble(coordinates[1]);
		p = new GeoPoint((int) (lat * 1E6), (int) (lng * 1E6));

		String friendCoordinates[] = { "31.025675", "121.428995" };
		double friendLat = Double.parseDouble(friendCoordinates[0]);
		double friendLng = Double.parseDouble(friendCoordinates[1]);
		pFriend = new GeoPoint((int) (friendLat * 1E6), (int) (friendLng * 1E6));

		mc.animateTo(p);
		mc.setZoom(16);
		// ---Add a location marker---
		MapOverlay mapOverlay = new MapOverlay();
		List<Overlay> listOfOverlays = mapView.getOverlays();
		listOfOverlays.clear();
		listOfOverlays.add(mapOverlay);
		mapView.invalidate();

		lm = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
		LocationListener locationListener = new MyLocationListener();
		lm.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0,
				locationListener);

		new Thread(new Runnable() {
			public void run() {

				while (true) {
					// String text = inputText.getText().toString();
					String str = getData();
					String[] separated = str.split(",");
					int latitudeE6 = Integer.parseInt(separated[0]);
					int longitudeE6 = Integer.parseInt(separated[1]);
					pFriend = new GeoPoint(latitudeE6, longitudeE6);

					mText.post(new Runnable() {
						public void run() {
							mText.setText("My friend's coordinates: "
									+ pFriend.toString());
						}
					});

					mapView.post(new Runnable() {
						public void run() {
							mapView.postInvalidate();
						}
					});
				}
			}
		}).start();

	}

	public String getData() {

		String result = "";

		if (!flagConnectServer) {
			try {
				server = new ServerSocket(4321);
				flagConnectServer = true;
			} catch (IOException e) {
				System.out.println("Could not listen on port 4321");
				flagConnectServer = false;
			}
		}

		if (!flagConnectClient) {

			try {
				client = server.accept();
				dataInputStream = new DataInputStream(client.getInputStream()); // read
																				// from
																				// client
																				// through
																				// inputstream
				dataOutputStream = new DataOutputStream(
						client.getOutputStream());// write to stream through
													// outputstream
				flagConnectClient = true;
			} catch (IOException e) {
				System.out.println("Accept failed: 4321");
				flagConnectClient = false;
			}
		}

		if (flagConnectClient) {
			try {
				dataOutputStream.writeUTF(p.toString());
				result = dataInputStream.readUTF();

			} catch (IOException e) {
				System.out.println("No I/O");
			}
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}

		if (result == null) {
			result = "failed";
		}
		return (result);
	}

	private class MyLocationListener implements LocationListener {
		@Override
		public void onLocationChanged(Location loc) {
			if (loc != null) {
				Toast.makeText(
						getBaseContext(),
						"Location changed : Lat: " + loc.getLatitude()
								+ " Lng: " + loc.getLongitude(),
						Toast.LENGTH_SHORT).show();
			}
			p = new GeoPoint((int) (loc.getLatitude() * 1E6),
					(int) (loc.getLongitude() * 1E6));
			mc.animateTo(p);
			mc.setZoom(18);
		}

		@Override
		public void onProviderDisabled(String provider) {
		}

		@Override
		public void onProviderEnabled(String provider) {
		}

		@Override
		public void onStatusChanged(String provider, int status, Bundle extras) {
		}

	}

	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}

}